
/*
 * async.c:
 *
 * Copyright (c) 1999 James McKenzie <james@fishsoup.dhs.org>.
 * All rights reserved.
 *
 * $Log: async.c,v $
 * Revision 1.4  2000/03/11 18:34:56  root
 * #make_checkin
 *
 * Revision 1.3  1999/11/21 02:56:20  root
 * #make_checkin
 *
 * Revision 1.2  1999/11/21 01:08:34  root
 * #make_checkin
 *
 * Revision 1.1  1999/11/20 18:47:30  root
 * #make_checkin
 *
 *
 */

static char rcsid[] = "$Id: async.c,v 1.4 2000/03/11 18:34:56 root Exp root $";

#include "project.h"

#if 1                           /*Linux version ? */

#include <sys/ioctl.h>
#include <sys/termios.h>

#ifndef LOCK_PREFIX
#define LOCK_PREFIX	"/var/lock/LCK.."
#endif

static char *
async_lock (char *dev)
{
  char *lock_file;
  char hdb_lock_buffer[12];
  int fd, n;
  int pid = getpid ();
  char *p;

  p = strrchr (dev, '/');
  if (p != NULL)
    {
      dev = ++p;
    }
  lock_file = digit_malloc (strlen (LOCK_PREFIX) + strlen (dev) + 1);
  strcpy (lock_file, LOCK_PREFIX);
  strcat (lock_file, dev);
  while (1)
    {
      fd = open (lock_file, O_EXCL | O_CREAT | O_RDWR, 0644);
      if (fd >= 0)
        {
          pid = getpid ();
#ifndef PID_BINARY
          sprintf (hdb_lock_buffer, "%010d\n", pid);
          write (fd, hdb_lock_buffer, 11);
#else
          write (fd, &pid, sizeof (pid));
#endif
          close (fd);
          return lock_file;
        }
      if (errno == EEXIST)
        {
          fd = open (lock_file, O_RDONLY, 0);
          if (fd < 0)
            {
              if (errno == ENOENT)
                {
                  continue;
                }
              break;
            }
          n = read (fd, hdb_lock_buffer, 11);
          close (fd);
          if (n < 0)
            {
              gripe (GRIPE_WARN, "Can't read pid from lock file %s", lock_file);
              break;
            }
          if (n > 0)
            {
#ifndef PID_BINARY
              hdb_lock_buffer[n] = '\0';
              sscanf (hdb_lock_buffer, " %d", &pid);
#else
              pid = ((int *) hdb_lock_buffer)[0];
#endif
              if (pid == 0 || pid == getpid ()
                  || (kill (pid, 0) == -1 && errno == ESRCH))
                {
                  n = 0;
                }
            }
          if (n == 0 && unlink (lock_file) == 0)
            {
              gripe (GRIPE_WARN, "Removed stale lock on %s (pid %d)",
                     dev, pid);
              continue;
            }
          gripe (GRIPE_WARN, "Device %s is locked by pid %d", dev, pid);
          break;
        }
      gripe (GRIPE_WARN, "Can't create lock file %s: %m(%d)", lock_file, errno);
      break;
    }

  free (lock_file);
  lock_file = NULL;
  return lock_file;
}

static void
async_unlock (char *lock_file)
{
  if (lock_file)
    {
      unlink (lock_file);
      free (lock_file);
    }
}

static struct speed
{
  int speed_int, speed_val;
}
speeds[] =
{

#ifdef B50
  {
    50, B50
  }
  ,
#endif
#ifdef B75
  {
    75, B75
  }
  ,
#endif
#ifdef B110
  {
    110, B110
  }
  ,
#endif
#ifdef B134
  {
    134, B134
  }
  ,
#endif
#ifdef B150
  {
    150, B150
  }
  ,
#endif
#ifdef B200
  {
    200, B200
  }
  ,
#endif
#ifdef B300
  {
    300, B300
  }
  ,
#endif
#ifdef B600
  {
    600, B600
  }
  ,
#endif
#ifdef B1200
  {
    1200, B1200
  }
  ,
#endif
#ifdef B1800
  {
    1800, B1800
  }
  ,
#endif
#ifdef B2000
  {
    2000, B2000
  }
  ,
#endif
#ifdef B2400
  {
    2400, B2400
  }
  ,
#endif
#ifdef B3600
  {
    3600, B3600
  }
  ,
#endif
#ifdef B4800
  {
    4800, B4800
  }
  ,
#endif
#ifdef B7200
  {
    7200, B7200
  }
  ,
#endif
#ifdef B9600
  {
    9600, B9600
  }
  ,
#endif
#ifdef B19200
  {
    19200, B19200
  }
  ,
#endif
#ifdef B38400
  {
    38400, B38400
  }
  ,
#endif
#ifdef B57600
  {
    57600, B57600
  }
  ,
#endif
#ifdef B115200
  {
    115200, B115200
  }
  ,
#endif
#ifdef EXTA
  {
    19200, EXTA
  }
  ,
#endif
#ifdef EXTB
  {
    38400, EXTB
  }
  ,
#endif
  {
    0, 0
  }
};

static int
translate_speed (int bps)
{
  struct speed *speedp;

  if (bps != 0)
    {
      for (speedp = speeds; speedp->speed_int; speedp++)
        {
          if (bps == speedp->speed_int)
            {
              return speedp->speed_val;
            }
        }
      gripe (GRIPE_WARN, "speed %d not supported", bps);
    }
  return 0;
}

static int
baud_rate_of (int speed)
{
  struct speed *speedp;

  if (speed != 0)
    {
      for (speedp = speeds; speedp->speed_int; speedp++)
        {
          if (speed == speedp->speed_val)
            {
              return speedp->speed_int;
            }
        }
    }
  return 0;
}


void
async_rts (Async a, int on)
{
  int modembits = TIOCM_RTS;
  ioctl (a->fd, (on ? TIOCMBIS : TIOCMBIC), &modembits);
}
void
async_dtr (Async a, int on)
{
  int modembits = TIOCM_DTR;
  ioctl (a->fd, (on ? TIOCMBIS : TIOCMBIC), &modembits);
}

int
async_setup (Async a, int speed, int bits, Async_Parity parity, int flow)
{
  struct termios tios;
  int spd;

  if (tcgetattr (a->fd, &tios) < 0)
    {
      gripe (GRIPE_WARN, "tcgetattr: %m(%d)", errno);
      return (0);
    }

  tios.c_cflag &= ~(CSIZE | CSTOPB | PARENB | CLOCAL);

  switch (parity)
    {
    case ASYNC_NONE:
      break;
    case ASYNC_EVEN:
      tios.c_cflag |= PARENB;
      break;
    case ASYNC_ODD:
      tios.c_cflag |= PARENB | PARODD;
      break;
    default:
      gripe (GRIPE_WARN, "async setup parity %d not supported", parity);
      return 0;
    }

      tios.c_cflag |= CLOCAL;

  switch (bits)
    {
#ifdef CS8
    case 8:
      tios.c_cflag |= CS8;
      break;
#endif
#ifdef CS8
    case 7:
      tios.c_cflag |= CS7;
      break;
#endif
#ifdef CS8
    case 6:
      tios.c_cflag |= CS6;
      break;
#endif
    default:
      gripe (GRIPE_WARN, "async setup %d bit words not supported", bits);
      return 0;
    }

  if (flow)
    {
      tios.c_cflag |= CRTSCTS;
    }

  tios.c_cflag |= CREAD | HUPCL;  /*Enable receiver hangup on close */
  tios.c_iflag = IGNBRK | IGNPAR;  /*Ignore parity errors and breaks */

  /* Set to read bytes as they arrive */

  tios.c_oflag = 0;
  tios.c_lflag = 0;
  tios.c_cc[VMIN] = 1;
  tios.c_cc[VTIME] = 0;




  spd = translate_speed (speed);
  if (spd)
    {
      cfsetospeed (&tios, spd);
      cfsetispeed (&tios, spd);
    }
  else
    {
      speed = cfgetospeed (&tios);
      if (speed == B0)
        {
          gripe (GRIPE_WARN, "Baud rate is 0; need explicit baud rate");
          return 0;
        }
    }

  if (tcsetattr (a->fd, TCSAFLUSH, &tios) < 0)
    {
      gripe (GRIPE_WARN, "tcsetattr: %m");
      return 0;
    }
  return 1;

}

void
async_carrsens (Async a, int doit)
{
  struct termios tios;

  if (tcgetattr (a->fd, &tios) < 0)
    {
      gripe (GRIPE_WARN, "tcgetattr: %m(%d)", errno);
      return;
    }
  if (doit)
    {
      gripe (GRIPE_WARN, "now respecting carrier");
      tios.c_cflag &= ~CLOCAL;
    }
  else
    {
      gripe (GRIPE_WARN, "now ignoring carrier");
      tios.c_cflag |= CLOCAL;
    }

  if (tcsetattr (a->fd, TCSADRAIN, &tios) < 0)
    {
      gripe (GRIPE_WARN, "tcsetattr: %m");
      return;
    }
}


static void
async_save_state (Async a)
{
  struct termios *tios;

  if (a->old)
    free (a->old);
  tios = digit_malloc (sizeof (struct termios));
  a->old = (void *) tios;


  if (tcgetattr (a->fd, tios) < 0)
    {
      gripe (GRIPE_WARN, "tcgetattr");
      free (a->old);
      a->old = Null;
    }

}
static void
async_restore_state (Async a)
{
  struct termios *tios = (struct termios *) a->old;

  if (!tios)
    {
      gripe (GRIPE_WARN, "async_restore_state called with no state");
      return;
    }
  if (tcsetattr (a->fd, TCSAFLUSH, tios) < 0)
    {
      gripe (GRIPE_WARN, "tcsetattr failed");
    }
}


#endif

void
async_destructor (Async a)
{
  if (a->old)
    {
      async_restore_state (a);
      free (a->old);
    }
  if (a->fd >= 0)
    close (a->fd);
  if (a->lock)
    async_unlock (a->lock);
  free (a);
}


Async
async_open (char *dev)
{
  Async a = CREATE_OBJ (Async);

  a->fd = -1;
  a->lock = async_lock (dev);

  if (!a->lock)
    {
      gripe (GRIPE_WARN, "can't lock %s", dev);
      Obj_Delete (a);
      return NULL;
    }

  a->fd = open (dev, O_RDWR | O_NDELAY | O_NOCTTY);

  if (a->fd < 0)
    {
      gripe (GRIPE_WARN, "can't open %s(%m)", dev);
	Obj_Delete(a);
	return Null;
    }

  fcntl (a->fd, F_SETFL, O_RDWR);

  async_save_state (a);


  return (a);
}

void
async_drain (Async a)
{
  tcdrain (a->fd);
  tcflush (a->fd, TCIOFLUSH);
}
int async_write_to_slow_device(Async a,char *buf,int l)
{
int ret=0;
while (l--) {
if (write(a->fd,buf,1)!=1) {
	return ret;
}
ret++;
buf++;
usleep(100000);
}
return ret;
}

