
/*
 * digitizer.c:
 *
 * Copyright (c) 1999 James McKenzie <james@fishsoup.dhs.org>.
 * All rights reserved.
 *
 * $Log: digitizer.c,v $
 * Revision 1.7  2000/03/11 18:34:56  root
 * #make_checkin
 *
 * Revision 1.6  1999/11/22 04:36:00  root
 * #make_checkin
 *
 * Revision 1.5  1999/11/21 05:26:48  root
 * #make_checkin
 *
 * Revision 1.4  1999/11/21 02:56:20  root
 * #make_checkin
 *
 * Revision 1.3  1999/11/21 01:08:34  root
 * #make_checkin
 *
 * Revision 1.2  1999/11/20 18:47:30  root
 * #make_checkin
 *
 * Revision 1.1  1999/11/20 13:43:30  root
 * #make_checkin
 *
 *
 */

static char rcsid[] = "$Id: digitizer.c,v 1.7 2000/03/11 18:34:56 root Exp root $";

#include "project.h"


static int callback_id = 1;


static void
digitizer_send_event (Digitizer d, Digitizer_Event * e)
{

  Digitizer_Callback cb = d->callbacks->head;


  while (cb)
    {
      (cb->func) (d, e, cb->ptr);
      cb = cb->next;
    }

}
static void
do_report (Digitizer d, float x, float y, int state)
{
  Digitizer_Event e;
  int i;


  e.x = x;
  e.y = y;
  e.state = state;
  e.button = 0;

  if (d->state == state)
    {

      if ((d->x != x) || (d->y != y))
        {
          d->x = x;
          d->y = y;

          e.type = (state & DIGIT_BUTTONS) ? DIGITIZER_DRAG : DIGITIZER_MOTION;
          digitizer_send_event (d, &e);
        }
      return;

    }

  if ((d->state & DIGIT_OUT_AREA) != (state & DIGIT_OUT_AREA))
    {
      e.type = (state & DIGIT_OUT_AREA) ? DIGITIZER_LEAVEACTIVE : DIGITIZER_ENTERACTIVE;
      d->state &= ~DIGIT_OUT_AREA;
      d->state |= state & DIGIT_OUT_AREA;
      digitizer_send_event (d, &e);
      return;
    }

  for (i = 0; i < DIGIT_NUMBUTTONS; ++i)
    {

      if ((d->state & DIGIT_BUTTON (i)) != (state & DIGIT_BUTTON (i)))
        {
          e.type = (state & DIGIT_BUTTON (i)) ? DIGITIZER_BUTTDOWN : DIGITIZER_BUTTUP;
          e.button = i;
          d->state &= ~DIGIT_BUTTON (i);
          d->state |= state & DIGIT_BUTTON (i);
          digitizer_send_event (d, &e);
          return;

        }


    }

}




static void
keyboard_init (Digitizer d)
{
  d->wholepage->a = d->wholepage->d = 0.1;
  d->x = d->y = 5;
  d->state = 0;
}

static void
keyboard_read (Digitizer d)
{
  Digitizer_Event e;
  guchar c;

  read (d->async->fd, &c, 1);

  if (d->state & DIGIT_BUTTONS)
    e.type = DIGITIZER_DRAG;
  else
    e.type = DIGITIZER_MOTION;

  e.button = 0;

  switch (c)
    {
    case 'l':
      d->x -= 1;
      break;
    case 'r':
      d->x += 1;
      break;
    case 'd':
      d->y -= 1;
      break;
    case 'u':
      d->y += 1;
      break;
    case 'a':
      d->state |= DIGIT_OUT_AREA;
      e.type = DIGITIZER_ENTERACTIVE;
      break;
    case 'A':
      d->state &= ~DIGIT_OUT_AREA;
      e.type = DIGITIZER_LEAVEACTIVE;
      break;
    case '1':
      d->state |= DIGIT_BUTTON (1);
      e.type = DIGITIZER_BUTTDOWN;
      e.button = 1;
      break;
    case '!':
      d->state &= ~DIGIT_BUTTON (1);
      e.type = DIGITIZER_BUTTUP;
      e.button = 1;
      break;
    }
  e.x = d->x;
  e.y = d->y;
  e.state = d->state;

  digitizer_send_event (d, &e);
}

static int
keyboard_ioctl (Digitizer d, Digitizer_Ioctl i, void *arg)
{


}

static int
keyboard_probe (Digitizer d)
{
  gripe (GRIPE_WARN, "keyboard_probe()");
  return 1;
}

static void
microsoft_init (Digitizer d)
{
  struct microsoft_private *p = &d->private.msft;
  async_rts (d->async, 1);
  async_dtr (d->async, 1);
  async_drain (d->async);
  async_setup (d->async, 1200, 7, ASYNC_NONE, 0);

  p->ax = 0;
  p->ay = 0;
  p->astate = 0;

  d->wholepage->a = d->wholepage->d = 1.0 / 11.5;  /*A4 */
  d->x = d->y = 5.0;
}

static void
microsoft_read (Digitizer d)
{
  struct microsoft_private *p = &d->private.msft;

  guchar c;
  int i;

  while (canreadfrom (d->async->fd))
    {
      read (d->async->fd, &c, 1);

      if (c & 0x40)
        p->state = 0;

      switch (p->state)
        {
        case 0:
          if (c & 0x20)
            {
              p->left = 1;
              p->astate |= DIGIT_BUTTON (0);
            }
          else
            {
              p->left = 0;
              p->astate &= ~DIGIT_BUTTON (0);
            }
          if (c & 0x10)
            {
              p->right = 1;
              p->astate |= DIGIT_BUTTON (2);
            }
          else
            {
              p->right = 0;
              p->astate &= ~DIGIT_BUTTON (2);
            }
#ifdef NOFAKEACTIVE
          p->astate &= ~DIGIT_OUT_AREA;
#endif

          p->x = (c & 0x3) << 6;
          p->y = (c & 0xc) << 4;
          p->state = 1;
          break;
        case 1:
          p->x |= (c & 0x3f);
          if (p->x & 0x80)
            p->x -= 256;
          p->ax += 0.01 * (float) p->x;
          p->state = 2;
          break;
        case 2:
          p->y |= (c & 0x3f);
          if (p->y & 0x80)
            p->y -= 256;
          p->ay -= 0.01 * (float) p->y;
          do_report (d, p->ax, p->ay, p->astate);
          p->state = 3;
          break;
        case 3:                /*Logitech stuff or fuck up */
#ifdef NOFAKEACTIVE
          if (c == 0x0)
            {
              p->middle = 0;
              p->astate &= ~DIGIT_BUTTON (1);

            }
          if (c == 0x20)
            {
              p->middle = 1;
              p->astate |= DIGIT_BUTTON (1);
            }
#else
          if (c == 0x20)
            {
              p->astate |= DIGIT_OUT_AREA;
            }
          if (c == 0x0)
            {
              p->astate &= ~DIGIT_OUT_AREA;
            }
#endif
          do_report (d, p->ax, p->ay, p->astate);
          p->state = 4;
          break;
        case 4:
        default:
          p->state = 4;
        }



    }
}


static int
microsoft_ioctl (Digitizer d, Digitizer_Ioctl i, void *arg)
{

}

static int
microsoft_probe (Digitizer d)
{

  microsoft_init (d);
  usleep (100000);
  async_drain (d->async);
  async_dtr (d->async, 0);
  usleep (100000);
  async_dtr (d->async, 1);
  usleep (100000);

/* Should now analyse stream for an 'M" */

  async_drain (d->async);


  microsoft_read (d);

}


static void
summabpone_read (Digitizer d)
{
  struct sixbyte_private *p = &d->private.sixbyte;

  guchar c;
  int x;
  int y;
  int state;

  while (canreadfrom (d->async->fd))
    {
      read (d->async->fd, &c, 1);
      c&=0x7f;

      if (c & 0x40)
        p->state = 0;

      switch (p->state)
        {
        case 0:
	  p->b0=c;
          p->state = 1;
          break;
        case 1:
	  p->b1=c;
          p->state = 2;
          break;
        case 2:
	  p->b2=c;
	  p->state=3;
	break;
        case 3:
	  p->b3=c;
          p->state = 4;
          break;
        case 4:
	  p->b4=c;
	  x=p->b1+(p->b2 << 6);  
	  y=p->b3+(p->b4 << 6);  


	  state=(p->b0 & 0x3c)>> 2;
          if (p->b0 & 1) state |= DIGIT_OUT_AREA;
	  
          do_report (d, (float) x, (float) y, state);
	  p->state=5;
          break;
        case 5:
        default:
          p->state = 5;
        }
    }
}


static void
summasketch_read (Digitizer d)
{
  struct sixbyte_private *p = &d->private.sixbyte;

  guchar c;
  int x;
  int y;
  int state;

  while (canreadfrom (d->async->fd))
    {
      read (d->async->fd, &c, 1);

      if (c & 0x80)
        p->state = 0;

      switch (p->state)
        {
        case 0:
	  p->b0=c;
          p->state = 1;
          break;
        case 1:
	  p->b1=c;
          p->state = 2;
          break;
        case 2:
	  p->b2=c;
	  p->state=3;
	break;
        case 3:
	  p->b3=c;
          p->state = 4;
          break;
        case 4:
	  p->b4=c;
	  x=p->b1+(p->b2 << 7);  
	  y=p->b3+(p->b4 << 7);  

	  state=(p->b0 & 0x7);

	  if (state) {
		state--;
		state=1<state;
	  }		

          if (p->b0 & 0x40) state |= DIGIT_OUT_AREA;
	  
          do_report (d, (float) x, (float) y, state);
	  p->state=5;
          break;
        case 5:
        default:
          p->state = 5;
        }
    }
}


static void
digipad_read (Digitizer d)
{
  struct sixbyte_private *p = &d->private.sixbyte;

  guchar c;
  int x;
  int y;
  int state;

  while (canreadfrom (d->async->fd))
    {
      read (d->async->fd, &c, 1);

      if (c & 0x80)
        p->state = 0;

      switch (p->state)
        {
        case 0:
	  p->b0=c;
          p->state = 1;
          break;
        case 1:
	  p->b1=c;
          p->state = 2;
          break;
        case 2:
	  p->b2=c;
	  p->state=3;
	break;
        case 3:
	  p->b3=c;
          p->state = 4;
          break;
        case 4:
	  p->b4=c;
	  p->state=5;
	break;
	case 5:
	  p->b5=c;
	
	  x=(p->b0 & 3) << 14;
 	  x|=p->b1 <<7;
 	  x|=p->b2;
	  y=(p->b3 & 3) << 14;
 	  y|=p->b4 << 7;
	  y|=p->b5;

	  if (p->b0 & 0x40) {
		state=0;
	  } else {
		state= (p->b0 & 0x3c)  >> 2;
		state=1<state;
	  }		

          do_report (d, (float) x, (float) y, state);
	  p->state=6;
          break;
        case 6:
        default:
          p->state = 6;
        }
    }
}



static void
kurta_read (Digitizer d)
{
  struct sixbyte_private *p = &d->private.sixbyte;

  guchar c;
  int x;
  int y;
  int state;

  while (canreadfrom (d->async->fd))
    {
      read (d->async->fd, &c, 1);

      if (c & 0x80)
        p->state = 0;

      switch (p->state)
        {
        case 0:
	  p->b0=c;
          p->state = 1;
          break;
        case 1:
	  p->b1=c;
          p->state = 2;
          break;
        case 2:
	  p->b2=c;
	  p->state=3;
	break;
        case 3:
	  p->b3=c;
          p->state = 4;
          break;
        case 4:
	  p->b4=c;
	  p->state=5;
	break;
	case 5:
	  p->b5=c;
	
	  x=(p->b0 & 3) << 14;
 	  x|=p->b1 <<7;
 	  x|=p->b2;
	  y=(p->b3 & 3) << 14;
 	  y|=p->b4 << 7;
	  y|=p->b5;

   	  state= (p->b0 & 0x3c)  >> 2;

          do_report (d, (float) x, (float) y, state);
	  p->state=6;
          break;
        case 6:
        default:
          p->state = 6;
        }
    }
}


static void
microgridIIasc_init (Digitizer d)
{
  struct ascii_private *p = &d->private.ascii;
  unsigned char reset[]="\033 \033Z";
  unsigned char init[]="\033M0\033C2\033I002\033R8\033A";

  async_rts (d->async, 1);
  async_dtr (d->async, 1);
  async_drain (d->async);
  async_setup (d->async, 9600, 7, ASYNC_EVEN, 0);

  async_write_to_slow_device(d->async,reset,sizeof(reset)-1);
  async_drain (d->async);
  sleep(1);
  async_write_to_slow_device(d->async,init,sizeof(init)-1);

	p->n=0;
	p->p=p->b=digit_malloc(DIGIT_ASCII_BUFLEN+1);
	p->b[DIGIT_ASCII_BUFLEN]=0;

  d->wholepage->a = d->wholepage->d = 1.0 / 60.0;  /*A0 */
  d->x=d->y=-1.0;
  d->state=0;
}

static int microgridIIasc_probe(Digitizer d)
{
return 1;
}

static int microgridIIasc_ioctl(Digitizer d, Digitizer_Ioctl i, void *arg)
{
return -1;
}

static void
microgridIIasc_read (Digitizer d)
{
  struct ascii_private *p = &d->private.ascii;

  float x,y;
  int state,astate;

  while (canreadfrom (d->async->fd))
    {
      if (p->n==DIGIT_ASCII_BUFLEN) {
	p->n=0;
	p->p=p->b;
      }

      if (read (d->async->fd, p->p, 1)==1) {
	if (*(p->p)==13) {
	*(p->p +1)=0;
	
	if (*(p->b)<32)  *(p->b)=32; /* Kill nl*/

	sscanf(p->b,"%f,%f,%d,%*d",&x,&y,&state);

	astate=0;
	if (state & 0x20) astate|=DIGIT_OUT_AREA;
	state&=0x1f;
	if (state>0) {
	astate|=1 <<(state-1);
	}

        do_report (d, x, y, astate);

	  p->n=0;
	  p->p=p->b;
	} else {
		p->p++;
		p->n++;
	}
	
        }

    }
}


static int
tabby_ioctl (Digitizer d, Digitizer_Ioctl i, void *arg)
{


}

static void
tabby_init (Digitizer d)
{
  struct tabby_private *p = &d->private.tabby;

  async_rts (d->async, 1);
  async_dtr (d->async, 1);
  async_drain (d->async);
  async_setup (d->async, 9600, 8, ASYNC_NONE, 0);


  d->wholepage->a = d->wholepage->d = 1.0 / 5.75 ;/*A5 */
  d->x=d->y=-1.0;
  d->state=0;
  p->state=0;
}

static void
tabby_read (Digitizer d)
{
  struct tabby_private *p = &d->private.tabby;

  guchar c;
  int i;
  float x,y;

  int state;
  while (canreadfrom (d->async->fd))
    {
	int i=0x80;

      read (d->async->fd, &c, 1);


      switch (p->state) {
	case 0:
	p->b1=c;
	p->state++;
	break;
	case 1:
	p->b2=c;
	p->state++;
	break;
	case 2:
	p->b3=c;
	p->state++;
	break;
	case 3:
	p->b4=c;
	p->state++;
	
	x=(float) (((p->b3 & 0xf) << 7) | (p->b1 & 0x7f));
	y=(float) (((p->b4 & 0xf) << 7) | (p->b2 & 0x7f));
	x=2047.0-x;
	x*=0.0027777;
	y*=0.0027777;

   	state=0;
	if (!(p->b3 & 0x20)) state|=DIGIT_BUTTON(0);
	if (!(p->b3 & 0x40)) state|=DIGIT_BUTTON(2);

	
	do_report(d,x,y,state);
	break;
	case 4:
	if (c&0x80) p->state=0;
	break;
      }

    }
}



static int
tabby_probe (Digitizer d)
{

  microsoft_init (d);
  usleep (100000);
  async_drain (d->async);
  async_dtr (d->async, 0);
  usleep (100000);
  async_dtr (d->async, 1);
  usleep (100000);


  async_drain (d->async);


  tabby_read (d);

}

static int
hipadplus_ioctl (Digitizer d, Digitizer_Ioctl i, void *arg)
{


}

static void
hipadplus_init (Digitizer d)
{
  struct hipad_private *p = &d->private.hipad;
  unsigned char setup[]={0x11,0x14,0x33,0x1b,0x14,'3','2','N','(',')','w','O','e','p',0x07};
  int i;

  async_rts (d->async, 1);
  async_dtr (d->async, 1);
  async_drain (d->async);
  async_setup (d->async, 9600, 8, ASYNC_NONE, 0);

  for (i=0;i<sizeof(setup);++i) {
	write(d->async->fd,&setup[i],1);
	usleep(50000);
  }
  

  d->wholepage->a = d->wholepage->d = 1.0 / 17.0 ;/*A3 */
  d->x=d->y=-1.0;
  d->state=0;
  p->state=0;
}

static void
hipadplus_read (Digitizer d)
{
  struct hipad_private *p = &d->private.hipad;

  guchar c;
  int i;
  float x,y;

  int state;
  while (canreadfrom (d->async->fd))
    {
	int i=0x80;

      read (d->async->fd, &c, 1);

      switch (p->state) {
	case 0:
	p->b1=c;
	if (c & 0x80) p->state++;
	break;
	case 1:
	p->b2=c;
	p->state++;
	break;
	case 2:
	p->b3=c;
	p->state++;
	break;
	case 3:
	p->b4=c;
	p->state++;
	break;
	case 4:
	p->b5=c;
	p->state++;
	break;
	case 5:
	p->b6=c;
	p->state=0;
	
	x=(float) (((p->b1 & 0x3) << 14 ) | ((p->b2 & 0x7f) << 7) | (p->b3 & 0x7f));
	y=(float) (((p->b4 & 0x3) << 14 ) | ((p->b5 & 0x7f) << 7) | (p->b6 & 0x7f));
	x*=1.0/2000.0;
	y*=1.0/2000.0;

   	state=0;
	if ((p->b1 & 0x04)) state|=DIGIT_BUTTON(0);
	if ((p->b1 & 0x08)) state|=DIGIT_BUTTON(2);

	
	do_report(d,x,y,state);
	break;
      }

    }
}



static int
hipadplus_probe (Digitizer d)
{

  hipadplus_init (d);
  usleep (100000);
  async_drain (d->async);
  async_dtr (d->async, 0);
  usleep (100000);
  async_dtr (d->async, 1);
  usleep (100000);


  async_drain (d->async);


}
typedef void (*Dt_np) (Digitizer);
typedef void (*Dt_rp) (Digitizer);
typedef int (*Dt_pp) (Digitizer);
typedef int (*Dt_ip) (Digitizer, Digitizer_Ioctl, void *);

static Dt_np init_procs[] =
{keyboard_init, microsoft_init,Null,Null,microgridIIasc_init,tabby_init,hipadplus_init};
static Dt_rp read_procs[] =
{keyboard_read, microsoft_read,Null,Null,microgridIIasc_read,tabby_read,hipadplus_read};
static Dt_pp probe_procs[] =
{keyboard_probe, microsoft_probe,Null,Null,microgridIIasc_probe,tabby_probe,hipadplus_probe};
static Dt_ip ioctl_procs[] =
{keyboard_ioctl, microsoft_ioctl,Null,Null,microgridIIasc_ioctl,tabby_ioctl,hipadplus_ioctl};

int
digitizer_register_eventproc (Digitizer d, Digitizer_EventProc f, void *ptr)
{
  Digitizer_Callback cb;

  cb = CREATE_OBJ (Digitizer_Callback);
  cb->id = callback_id++;
  cb->func = f;
  cb->ptr = ptr;

  Obj_Insert (d->callbacks, cb, ObjInsertTail);

  return cb->id;
}

int
digitizer_unregister_eventproc (Digitizer d, int id)
{
  Digitizer_Callback cb = d->callbacks->head;

  while (cb)
    {

      if (cb->id == id)
        {
          Obj_Remove (d->callbacks, cb);
          Obj_Delete (cb);
          return 1;
        }

      cb = cb->next;
    }


  gripe (GRIPE_WARN, "digitizer_unregister_eventproc: can't find id %d\n", id);

  return 0;
}


int
digitizer_ioctl (Digitizer d, Digitizer_Ioctl i, void *arg)
{
  return ioctl_procs[d->type] (d, i, arg);
}

void
digitizer_fd_notify (Digitizer d)
{

  if (!canreadfrom (d->async->fd))
    return;

  read_procs[d->type] (d);

}

#define PORT "/dev/digitizer"

Digitizer
digitizer_create ()
{
  Digitizer d = (Digitizer) malloc (sizeof (struct digitizer_struct));

  d->async = async_open (PORT);


  if (!d->async) {
	gripe(GRIPE_WARN,"Failed to open serial port %s\n",PORT);
	digit_free(d);
	return Null;
  }

//  d->type = DIGITIZER_SUMMAUGRIDII;
//  d->type=DIGITIZER_TABBY;
  d->type=DIGITIZER_HIPADPLUS;

  d->wholepage = transform_create ();
  transform_null (d->wholepage);


  d->callbacks = CREATE_LIST (Digitizer_Callback);

  probe_procs[d->type] (d);
  init_procs[d->type] (d);

  return d;

}

void
digitizer_destroy (Digitizer d)
{
  Obj_Delete (d->callbacks);
  transform_free (d->wholepage);
  Obj_Delete (d->async);
  free (d);
}
