/*
 * digitizer.h:
 *
 * Copyright (c) 1999 James McKenzie <james@fishsoup.dhs.org>.
 *
 * $Id: digitizer.h,v 1.5 2000/03/11 18:34:56 root Exp root $
 *
 * $Log: digitizer.h,v $
 * Revision 1.5  2000/03/11 18:34:56  root
 * #make_checkin
 *
 * Revision 1.4  1999/11/21 02:56:20  root
 * #make_checkin
 *
 * Revision 1.3  1999/11/21 01:08:34  root
 * #make_checkin
 *
 * Revision 1.2  1999/11/20 18:47:30  root
 * #make_checkin
 *
 * Revision 1.1  1999/11/20 13:43:30  root
 * #make_checkin
 *
 *
 */

#ifndef __DIGITIZER_H_          /* include guard */
#define __DIGITIZER_H_


typedef enum
  {
    DIGITIZER_KEYBOARD = 0,
    DIGITIZER_MICROSOFT = 1,
    DIGITIZER_MOUSESYS  = 2,
    DIGITIZER_SUMMABPONE =3,
    DIGITIZER_SUMMAUGRIDII =4,
    DIGITIZER_TABBY=5,
    DIGITIZER_HIPADPLUS=6,
  }
Digitizer_Type;

typedef enum
  {
    DIGITIZER_MOTION = 1,
    DIGITIZER_DRAG = 2,
    DIGITIZER_BUTTDOWN = 3,
    DIGITIZER_BUTTUP = 4,
    DIGITIZER_ENTERACTIVE = 5,
    DIGITIZER_LEAVEACTIVE = 6
  }
Digitizer_Event_Type;


typedef enum
  {
    DIG_STOP = 0,
    DIG_START = 1,
    DIG_RESET = 2
  }
Digitizer_Ioctl;

#define DIGIT_BUTTON(n)		(1L << (n))
#define DIGIT_BUTTONS		0x1ffff
#define DIGIT_NUMBUTTONS	17
#define DIGIT_OUT_AREA		(1L << 17)

typedef struct
  {
    int type;
    int button;
    float x, y;
    int state;
  }
Digitizer_Event;



struct keyboard_private
  {
    int dummy;
  };

struct microsoft_private
  {
    int state;

    int left, right;
    int x;
    int y;
    int middle;

    float ax;
    float ay;
    int astate;
  };
struct mousesys_private
  {
    int state;
    int left,right,middle;
    int x;
    int y;

    float ax;
    float ay;
    int astate;
  };
struct tabby_private
{
    int state;
    int b0,b1,b2,b3,b4,b5;
};
struct sixbyte_private
{
    int state;
    int b0,b1,b2,b3,b4,b5;
};
struct hipad_private
{
    int state;
    int b1,b2,b3,b4,b5,b6;
};
#define DIGIT_ASCII_BUFLEN 511
struct ascii_private 
{
    int n;
    unsigned char *b;
    unsigned char *p;
};

struct digitizer_struct;

typedef void (*Digitizer_EventProc) (struct digitizer_struct *, Digitizer_Event *, void *);

BEGIN_OBJECT (Digitizer_Callback)
     Digitizer_EventProc func;
     void *ptr;
     int id;
END_OBJECT (Digitizer_Callback)

CREATOR (Digitizer_Callback) = Obj_DefaultCreator;
DESTRUCTOR (Digitizer_Callback) = Obj_DefaultDestructor;

     typedef struct digitizer_struct
       {
         Async async;
         Digitizer_Type type;   /*Type of digitizer */

         union
           {
             struct keyboard_private keyb;
             struct microsoft_private msft;
	     struct sixbyte_private sixbyte;
	     struct ascii_private ascii;
	     struct tabby_private tabby;
	     struct hipad_private hipad;

           }
         private;

         float x, y;            /*Last known state of digitizer */
         int state;

         Transform wholepage;

         Digitizer_Callback_list callbacks;  /* Callback lists */

       }
 *Digitizer;

#endif /* __DIGITIZER_H_ */
