/*
 * draw_window.c:
 *
 * Copyright (c) 1999 James McKenzie <james@fishsoup.dhs.org>.
 * All rights reserved.
 *
 * $Log: draw_window.c,v $
 * Revision 1.14  2000/03/11 18:34:56  root
 * #make_checkin
 *
 * Revision 1.13  1999/11/22 04:43:40  root
 * #make_checkin
 *
 * Revision 1.12  1999/11/22 04:36:00  root
 * #make_checkin
 *
 * Revision 1.11  1999/11/21 19:48:05  root
 * #make_checkin
 *
 * Revision 1.10  1999/11/21 19:23:06  root
 * #make_checkin
 *
 * Revision 1.9  1999/11/21 05:26:48  root
 * #make_checkin
 *
 * Revision 1.8  1999/11/21 02:56:20  root
 * #make_checkin
 *
 * Revision 1.7  1999/11/21 01:08:34  root
 * #make_checkin
 *
 * Revision 1.6  1999/11/20 18:47:30  root
 * #make_checkin
 *
 * Revision 1.5  1999/11/20 13:43:30  root
 * #make_checkin
 *
 * Revision 1.4  1999/11/19 00:47:39  root
 * #make_checkin
 *
 * Revision 1.3  1999/11/19 00:27:46  root
 * #make_checkin
 *
 * Revision 1.2  1999/11/19 00:26:09  root
 * #make_checkin
 *
 * Revision 1.1  1999/11/19 00:25:33  root
 * #make_checkin
 *
 *
 */

static char rcsid[] = "$Id: draw_window.c,v 1.14 2000/03/11 18:34:56 root Exp root $";

#include "project.h"


static void
do_color (Draw_Window dw, GdkColor * c, int r, int g, int b)
{
  c->red = r;
  c->green = g;
  c->blue = b;
  if (!gdk_color_alloc (dw->cmap, c))
    {
      gripe (GRIPE_WARN, "Couldn't allocate colour %d/%d/%d\n", r, g, b);
    }
}
static void
checkgc (Draw_Window dw)
{
  if (dw->gc)
    return;
  dw->gc = gdk_gc_new (dw->area->window);

  dw->cmap = gdk_colormap_get_system ();
  do_color (dw, &dw->red, 0xffff, 0, 0);
  do_color (dw, &dw->green, 0x0, 0xffff, 0);
  do_color (dw, &dw->blue, 0, 0, 0xffff);
  do_color (dw, &dw->cyan, 0, 0xffff, 0xffff);
  do_color (dw, &dw->magenta, 0xffff, 0, 0xffff);
  do_color (dw, &dw->yellow, 0xffff, 0xffff, 0);
  do_color (dw, &dw->gray, 0x7fff, 0x7fff, 0x7fff);
  dw->white = dw->area->style->black;
  dw->white = dw->area->style->white;
}

static void
redo_transforms (Draw_Window dw)
{
  transform_concat (dw->t, dw->wt, dw->vt);
  transform_invert (dw->it, dw->t);

}

static void
sortout_rulers (Draw_Window dw)
{
  float xl, xh;
  float yl, yh;
  float w = (float) dw->area->allocation.width;
  float h = (float) dw->area->allocation.height;

  xl = transform_x (dw->it, 0, 0);
  yl = transform_y (dw->it, 0, 0);
  xh = transform_x (dw->it, w, h);
  yh = transform_y (dw->it, w, h);


  gtk_ruler_set_range (GTK_RULER (dw->hrule), xl, xh, 0, xh);
  gtk_ruler_set_range (GTK_RULER (dw->vrule), yl, yh, 0, yh);

}

static void
sortout_windowsize (Draw_Window dw)
{
  transform_free (dw->wt);
  dw->wt = transform_window ((float) dw->area->allocation.width, (float) dw->area->allocation.height);
  redo_transforms (dw);
  sortout_rulers (dw);
}

static void
draw_line (Draw_Window dw, Line l)
{
  float fx, fy;
  Point *p = l->l;
  int n = l->n;

  checkgc (dw);

  if (l->type == LINE_LINE)
    {
      GdkPoint *gp = digit_malloc (sizeof (GdkPoint) * l->n);
      GdkPoint *gptr = gp;

      while (n--)
        {
          gptr->x = (int) transform_x (dw->t, p->x, p->y);
          gptr->y = (int) transform_y (dw->t, p->x, p->y);
          p++;
          gptr++;
        }

      gdk_draw_lines (dw->area->window, dw->gc,
                      gp, l->n);

      digit_free (gp);
    }
  else
    {
      int x = (int) transform_x (dw->t, p->x, p->y);
      int y = (int) transform_y (dw->t, p->x, p->y);


#define FISH 5
      gdk_draw_line (dw->area->window, dw->gc, x - FISH, y - FISH, x + FISH, y - FISH);
      gdk_draw_line (dw->area->window, dw->gc, x + FISH, y - FISH, x + FISH, y + FISH);
      gdk_draw_line (dw->area->window, dw->gc, x + FISH, y + FISH, x - FISH, y + FISH);
      gdk_draw_line (dw->area->window, dw->gc, x - FISH, y + FISH, x - FISH, y - FISH);

      gdk_draw_line (dw->area->window, dw->gc, x - FISH, y - FISH, x + FISH, y + FISH);
      gdk_draw_line (dw->area->window, dw->gc, x + FISH, y - FISH, x - FISH, y + FISH);



    }
}

draw_endofline (Draw_Window dw, Line l)
{
  int x1, y1, x2, y2;
  Point *p = l->l;
  if (l->n < 2)
    return;
  checkgc (dw);

  if (l->type != LINE_LINE)
    return;

  p += (l->n - 2);

  x1 = (int) transform_x (dw->t, p->x, p->y);
  y1 = (int) transform_y (dw->t, p->x, p->y);
  p++;
  x2 = (int) transform_x (dw->t, p->x, p->y);
  y2 = (int) transform_y (dw->t, p->x, p->y);

  gdk_draw_line (dw->area->window, dw->gc,
                 x1, y1, x2, y2);

}

setgc_lines (Draw_Window dw)
{
  checkgc (dw);
  gdk_gc_set_function (dw->gc, GDK_COPY);
  gdk_gc_set_foreground (dw->gc, &dw->black);
}
setgc_current (Draw_Window dw)
{
  checkgc (dw);
  gdk_gc_set_function (dw->gc, GDK_COPY);
  gdk_gc_set_foreground (dw->gc, &dw->red);
}
setgc_selected (Draw_Window dw)
{
  checkgc (dw);
  gdk_gc_set_function (dw->gc, GDK_COPY);
  gdk_gc_set_foreground (dw->gc, &dw->magenta);
}
setgc_background (Draw_Window dw)
{
  checkgc (dw);
  gdk_gc_set_function (dw->gc, GDK_COPY);
  gdk_gc_set_foreground (dw->gc, &dw->gray);
}

setgc_active (Draw_Window dw)
{
  checkgc (dw);
  gdk_gc_set_function (dw->gc, GDK_COPY);
  gdk_gc_set_foreground (dw->gc, &dw->white);
}


static void
draw_active (Draw_Window dw)
{
  Transform t = transform_create ();

  transform_invert (t, dw->drawing->d->wholepage);


  {
    float fx = transform_x (t, 1.0, 1.0);
    float fy = transform_y (t, 1.0, 1.0);
    int x1 = transform_x (dw->t, 0, 0);
    int x2 = transform_x (dw->t, fx, fy);
    int y1 = transform_y (dw->t, 0, 0);
    int y2 = transform_y (dw->t, fx, fy);
    int t;

    checkgc (dw);

    if (x2 < x1)
      {
        t = x1;
        x1 = x2;
        x2 = t;
      }

    if (y2 < y1)
      {
        t = y1;
        y1 = y2;
        y2 = t;
      }

    x2 -= x1;
    y2 -= y1;


    setgc_active (dw);
    gdk_draw_rectangle (dw->area->window, dw->gc, 1, x1, y1, x2, y2);

  }
}
static void
do_repaint (Draw_Window dw)
{
  Drawing d = dw->drawing;
  Line l = d->lines->head;

  redo_transforms (dw);
  sortout_rulers (dw);

  checkgc (dw);

  setgc_background (dw);
  gdk_draw_rectangle (dw->area->window, dw->gc, 1, 0, 0,
                   dw->area->allocation.width, dw->area->allocation.height);

  draw_active (dw);



  setgc_lines (dw);
  while (l)
    {
      draw_line (dw, l);
      l = l->next;
    }

  setgc_current (dw);
  if (d->current)
    draw_line (dw, d->current);

  setgc_selected (dw);
  if (d->selected)
    draw_line (dw, d->selected);

  dw->curs_on = False;
  dw->zoomb_on = False;

}
static void
draw_zoombox (Draw_Window dw)
{
  gdk_gc_set_function (dw->gc, GDK_INVERT);

  gdk_draw_rectangle (dw->area->window, dw->gc, 0,
                      dw->zoomb_x, dw->zoomb_y,
                      dw->zoomb_w, dw->zoomb_h);


}

static void
hide_zoombox (Draw_Window dw)
{
  if (dw->zoomb_on)
    {
      draw_zoombox (dw);
      dw->zoomb_on = False;
    }
}


static void
move_zoombox (Draw_Window dw, float x, float y)
{
  int x1, y1;
  int x2, y2;
  int t;

  hide_zoombox (dw);

  x1 = transform_x (dw->t, dw->zoom_x, dw->zoom_y);
  y1 = transform_y (dw->t, dw->zoom_x, dw->zoom_y);
  x2 = transform_x (dw->t, x, y);
  y2 = transform_y (dw->t, x, y);

  if (x2 < x1)
    {
      t = x1;
      x1 = x2;
      x2 = t;
    }

  if (y2 < y1)
    {
      t = y1;
      y1 = y2;
      y2 = t;
    }

  dw->zoomb_x = x1;
  dw->zoomb_y = y1;
  dw->zoomb_w = x2 - x1;
  dw->zoomb_h = y2 - y1;

  draw_zoombox (dw);
  dw->zoomb_on = True;

}

static void
do_mouse_inject (Draw_Window dw, int x, int y, int state)
{
  float fx, fy, ax, ay;
  int astate;

  fx = (float) x;
  fy = (float) y;

  astate = 0;

  ax = transform_x (dw->it, fx, fy);
  ay = transform_y (dw->it, fx, fy);

  if (state & 0x100)
    astate |= MOUSE_BUTTON (0);
  if (state & 0x200)
    astate |= MOUSE_BUTTON (1);
  if (state & 0x400)
    astate |= MOUSE_BUTTON (2);

  switch (dw->zooming)
    {
    case 0:
      mouse_inject_state (dw->mouse, ax, ay, astate);
      break;
    case 1:                    /*Zoom requested */
      if (astate & MOUSE_BUTTON (0))
        {
          dw->zoom_x = ax;
          dw->zoom_y = ay;
          dw->zooming = 2;
        }
      break;
    case 2:                    /*Zoom in progress */
      if (astate & MOUSE_BUTTON (0))
        {
          move_zoombox (dw, ax, ay);
        }
      else
        {
          transform_from_zoom (dw->vt, dw->zoom_x, dw->zoom_y, ax, ay);
          dw->zooming = 0;
          do_repaint (dw);

        }
      break;
    }

}


static gint
button_event (GtkWidget * widget,
              GdkEventButton * event, gpointer ptr)
{
  Draw_Window dw = (Draw_Window) ptr;
  int x, y;
  GdkModifierType state;

  gdk_window_get_pointer (event->window, &x, &y, &state);

  do_mouse_inject (dw, x, y, state);
  return TRUE;
}

static void
draw_cursor (Draw_Window dw)
{
  checkgc (dw);

  gdk_gc_set_function (dw->gc, GDK_INVERT);

  gdk_draw_line (dw->area->window, dw->gc,
                 dw->curs_x - 10, dw->curs_y,
                 dw->curs_x + 10, dw->curs_y);

  gdk_draw_line (dw->area->window, dw->gc,
                 dw->curs_x, dw->curs_y - 10,
                 dw->curs_x, dw->curs_y + 10);
}


static void
remove_cursor (Draw_Window dw)
{
  if (!dw->curs_on)
    return;

  draw_cursor (dw);
  dw->curs_on = False;
}

static void
replace_cursor (Draw_Window dw)
{
  if (dw->curs_on)
    return;
  draw_cursor (dw);
  dw->curs_on = True;
}


static void
place_cursor (Draw_Window dw, int x, int y)
{

  dw->curs_x = x;
  dw->curs_y = y;

  draw_cursor (dw);

  dw->curs_on = True;
}


static void
notify_cursor (Draw_Window dw, int x, int y, int not_active)
{
  GdkEventMotion event;

  event.x = x;
  event.y = y;
  event.is_hint = 0;

/*FIXME: */

  ((GtkSignalFunc) GTK_WIDGET_CLASS (GTK_OBJECT (dw->hrule)->klass)->motion_notify_event) (dw->hrule, &event, (gpointer) GTK_OBJECT (dw->hrule));

  ((GtkSignalFunc) GTK_WIDGET_CLASS (GTK_OBJECT (dw->vrule)->klass)->motion_notify_event) (dw->vrule, &event, (gpointer) GTK_OBJECT (dw->vrule));

  remove_cursor (dw);
  if (!not_active)
    place_cursor (dw, x, y);



}

static void
puck_motion (Digitizer d, Digitizer_Event * e, void *ptr)
{
  Draw_Window dw = (Draw_Window) ptr;
  int a = e->state & DIGIT_OUT_AREA;

/* x and y in window coords */
  int x = (int) transform_x (dw->t, e->x, e->y);
  int y = (int) transform_y (dw->t, e->x, e->y);


  notify_cursor (dw, x, y, a);

}


static gint
motion_notify_event (GtkWidget * widget,
                     GdkEventMotion * event,
                     gpointer ptr)
{
  int x, y;
  Draw_Window dw = (Draw_Window) ptr;
  GdkModifierType state;

  if (event->is_hint)
    gdk_window_get_pointer (event->window, &x, &y, &state);
  else
    {
      x = event->x;
      y = event->y;
      state = event->state;
    }

  do_mouse_inject (dw, x, y, state);

  return TRUE;
}


static gint
configure_event (
                  GtkWidget * w,
                  GdkEventConfigure * e, gpointer ptr)
{
  Draw_Window dw = (Draw_Window) ptr;
  sortout_windowsize (dw);
}



static gint
expose_event (GtkWidget * widget,
              GdkEventExpose * event, gpointer ptr)
{
  Draw_Window dw = (Draw_Window) ptr;

  do_repaint (dw);


  return FALSE;
}

static void
hide_annotate (Draw_Window dw)
{
  dw->acb_lock++;
  gtk_entry_set_text (GTK_ENTRY (dw->annotation), "");
  gtk_widget_set_sensitive (dw->annotation, 0);
  dw->acb_lock--;
}

static void
show_annotate (Draw_Window dw)
{
  if (!dw->drawing->selected)
    {
      hide_annotate (dw);
      return;
    }
  gtk_entry_set_editable (GTK_ENTRY (dw->annotation), 1);
  gtk_widget_set_sensitive (dw->annotation, 1);


  if (dw->drawing->selected->annot)
    {
      gtk_entry_set_text (GTK_ENTRY (dw->annotation), dw->drawing->selected->annot);
    }
  else
    {
      gtk_entry_set_text (GTK_ENTRY (dw->annotation), "");
    }
}


static void
drawing_event (Drawing dr, Drawing_Event * e, void *ptr)
{
  Draw_Window dw = (Draw_Window) ptr;
  int cs = dw->curs_on;

  if (cs)
    remove_cursor (dw);

  switch (e->type)
    {
    case DRAW_LINEDONE:
      setgc_lines (dw);
      draw_line (dw, e->d.l);
      break;
    case DRAW_NEWLINE:
      break;
    case DRAW_LINEADD:
      setgc_current (dw);
      draw_endofline (dw, e->d.l);
      break;
    case DRAW_SELLINE:
      show_annotate (dw);
      setgc_selected (dw);
      draw_line (dw, e->d.l);
      break;
    case DRAW_UNSELLINE:
      hide_annotate (dw);
      setgc_lines (dw);
      draw_line (dw, e->d.l);
      break;
    case DRAW_DELLINE:
      setgc_active (dw);
      hide_annotate (dw);
      draw_line (dw, e->d.l);
      break;
    case DRAW_NEWDRAWING:
      do_repaint (dw);
      break;

    }
  if (cs)
    replace_cursor (dw);


}

static void
dw_request_zoom (gpointer ptr, guint act, GtkWidget * w)
{
  Draw_Window dw = (Draw_Window) ptr;
  dw->zooming = True;
}

static void
dw_unzoom (gpointer ptr, guint act, GtkWidget * w)
{
  Draw_Window dw = (Draw_Window) ptr;
  transform_copy (dw->vt, dw->drawing->d->wholepage);
  do_repaint (dw);
}

static void
dw_redraw (gpointer ptr, guint act, GtkWidget * w)
{
  Draw_Window dw = (Draw_Window) ptr;
  do_repaint (dw);
}

static void
dw_delete (gpointer ptr, guint act, GtkWidget * w)
{
  Draw_Window dw = (Draw_Window) ptr;
  drawing_delete_selected (dw->drawing);
}
static void
annotate_callback (GtkWidget * widget, gpointer ptr)
{
  Draw_Window dw = (Draw_Window) ptr;
  Line s = dw->drawing->selected;
  gchar *c;

  if (dw->acb_lock)
    return;
  dw->acb_lock++;

  c = gtk_entry_get_text (GTK_ENTRY (dw->annotation));


  if (s)
    {

      if (s->annot)
        digit_free (s->annot);
      s->annot = strdup (c);
    }
  else
    {
      hide_annotate (dw);
    }
  dw->acb_lock--;
}

static void
dw_do_new (Draw_Window dw)
{
  drawing_flush (dw->drawing);
}

int
dw_do_save (Draw_Window dw, char *name)
{
  FILE *f = fopen (name, "w");

  if (!f)
    {
      gripe (GRIPE_WARN, "Can't open %s for writing", name);
      return 0;
    }

  magic_write (DIGIT_MAGIC, f);

  drawing_write (dw->drawing, f);
  magic_write (EOF_MAGIC, f);
  fclose (f);

  return 1;
}
int
dw_do_load (Draw_Window dw, char *name)
{
  FILE *f = fopen (name, "r");
  if (!f)
    {
      gripe (GRIPE_WARN, "Can't open %s for reading", name);
      return 0;
    }

  if (!magic_check (DIGIT_MAGIC, f))
    {
      gripe (GRIPE_WARN, "%s: wrong magic number", name);
      fclose (f);
      return 0;
    }

  drawing_load (dw->drawing, f);
  if (!magic_check (EOF_MAGIC, f))
    {
      gripe (GRIPE_WARN, "%s: wrong termination", name);
      dw_do_new (dw);
      fclose (f);
      return 0;
    }

  fclose (f);
  return 1;
}

int
dw_do_export (Draw_Window dw, char *name)
{
  FILE *f = fopen (name, "w");
  Line l;

  if (!f)
    {
      gripe (GRIPE_WARN, "Can't open %s for writing", name);
      return 0;
    }

  l = dw->drawing->lines->head;
  while (l)
    {
      if (l->type == LINE_POINT)
        {
          fprintf (f, "%12.5f %12.5f %s\n\n", l->l->x, l->l->y, l->annot ? l->annot : "");
        }
      l = l->next;
    }

  l = dw->drawing->lines->head;
  while (l)
    {
      if (l->type == LINE_LINE)
        {
          Point *p = l->l;
          int n = l->n;
          if (n)
            {
              fprintf (f, "%12.5f %12.5f %s\n", p->x, p->y, l->annot ? l->annot : "");
              n--;
              p++;
            }
          while (n--)
            {
              fprintf (f, "%12.5f %12.5f\n", p->x, p->y);
              p++;
            }
            fprintf (f, "%12.5f %12.5f\n",-333.0,-333.0);
        }
      l = l->next;
    }

              fprintf (f, "%12.5f %12.5f\n",-999.0,-999.0);

  fclose (f);
  return 1;
}


static void
dw_fs_ok (GtkWidget * w, gpointer ptr)
{
  Draw_Window dw = (Draw_Window) ptr;
  (dw->fscb) (w, ptr);
}

static void
dw_fs_cancel (GtkWidget * w, gpointer ptr)
{
  Draw_Window dw = (Draw_Window) ptr;
  gtk_window_set_title (GTK_WINDOW (dw->fileselector), "");
  dw->fscb = (GtkSignalFunc) dw_fs_cancel;
  gtk_widget_hide (dw->fileselector);

}

static gint
dw_fs_delete_cancel (GtkWidget * w, GdkEvent * e, gpointer ptr)
{
  dw_fs_cancel (w, ptr);
  return TRUE;
}

static void
dw_new (gpointer ptr, guint act, GtkWidget * w)
{
  Draw_Window dw = (Draw_Window) ptr;
  if (dw->drawing->lines->head != NULL)
    {
    if (modal_button_dialog_box("Discard current?","Yes","Cancel")!=MBDB_OK) return;
    }

  dw_do_new (dw);

}

static void dw_open2 (GtkWidget *, gpointer);
static void dw_save (gpointer, guint, GtkWidget *);
static void dw_export (gpointer, guint, GtkWidget *);


static void
dw_fs_load (GtkWidget * w, gpointer ptr)
{
  Draw_Window dw = (Draw_Window) ptr;
  char *n = gtk_file_selection_get_filename (GTK_FILE_SELECTION (dw->fileselector));

  if (dw_do_load (dw, n)) {
  dw_fs_cancel (w, ptr);
  } 
}
static void
dw_fs_save (GtkWidget * w, gpointer ptr)
{
  Draw_Window dw = (Draw_Window) ptr;
  char *n = gtk_file_selection_get_filename (GTK_FILE_SELECTION (dw->fileselector));
  dw_fs_cancel (w, ptr);

  if (!dw_do_save (dw, n))
    dw_save (ptr, 0, w);
}
static void
dw_fs_export (GtkWidget * w, gpointer ptr)
{
  Draw_Window dw = (Draw_Window) ptr;
  char *n = gtk_file_selection_get_filename (GTK_FILE_SELECTION (dw->fileselector));
  dw_fs_cancel (w, ptr);

  if (!dw_do_export (dw, n))
    dw_export (ptr, 0, w);
}

static void
dw_open (gpointer ptr, guint act, GtkWidget * w)
{
  Draw_Window dw = (Draw_Window) ptr;

  if (dw->drawing->lines->head != NULL)
    {
    if (modal_button_dialog_box("Discard current?","Yes","Cancel")!=MBDB_OK) return;
    }

  gtk_window_set_title (GTK_WINDOW (dw->fileselector), "File: Open");
  dw->fscb = dw_fs_load;
  gtk_widget_show (dw->fileselector);
}

static void
dw_save (gpointer ptr, guint act, GtkWidget * w)
{
  Draw_Window dw = (Draw_Window) ptr;

  gtk_window_set_title (GTK_WINDOW (dw->fileselector), "File: Save");
  dw->fscb = dw_fs_save;
  gtk_widget_show (dw->fileselector);

}
static void
dw_export (gpointer ptr, guint act, GtkWidget * w)
{
  Draw_Window dw = (Draw_Window) ptr;

  gtk_window_set_title (GTK_WINDOW (dw->fileselector), "File: Export");
  dw->fscb = dw_fs_export;
  gtk_widget_show (dw->fileselector);

}

static void
dw_about (gpointer ptr, guint act, GtkWidget * w)
{
  Draw_Window dw = (Draw_Window) ptr;
  char buf[1024];
  snprintf(buf,sizeof(buf)-1,  "Digit: (c) 1999 James McKenzie\n"
    "<james@fishsoup.dhs.org>\n"
    "Version:\n"
    "%s\n"
    "\n"
    "ABSOLUTELY NO WARRANTY WHATSOEVER - NO IMPLIED WARRANTY\n"
    "- REALLY REALLY NO WARRANTY - YOUR COMPUTER MAY EXPLODE\n"
    "YOUR DIGITIZING TABLE MAY LAND ON YOUR FOOT - TOUGH\n"
    "If you digitize a map with this and end up bombing the \n"
    "Chinese embassy then don't blame me.\n",rcsid);


  modal_button_dialog_box(buf,"Ok",Null);

}

void
dw_quit (Draw_Window dw)
{

  if (dw->drawing->lines->head != NULL)
    {

    if (modal_button_dialog_box("Really Quit?","Yes","Cancel")!=MBDB_OK) return;
    }
  gtk_main_quit ();
}


static void
dw_menu_quit (gpointer ptr, guint act, GtkWidget * w)
{
  Draw_Window dw = (Draw_Window) ptr;
  dw_quit (dw);
}


static GtkItemFactoryEntry menu_items[] =
{
/*     Name      Key      Callback    ?  Type */
  {"/_File", NULL, NULL, 0, "<Branch>"},
  {"/File/_New", "<control>N", dw_new, 0, NULL},
  {"/File/_Open", "<control>O", dw_open, 0, NULL},
  {"/File/_Save", "<control>S", dw_save, 0, NULL},
  {"/File/_Export As", "<control>E", dw_export, 0, NULL},
  {"/File/sep1", NULL, NULL, 0, "<Separator>"},
  {"/File/Quit", "<control>Q", dw_menu_quit, 0, NULL},
  {"/_Edit", NULL, NULL, 0, "<Branch>"},
  {"/Edit/Delete", "<control>D", dw_delete, 0, NULL},
  {"/_View", NULL, NULL, 0, "<Branch>"},
  {"/View/Redraw", "<control>W", dw_redraw, 0, NULL},
  {"/View/Fit page", "<control>F", dw_unzoom, 0, NULL},
  {"/View/Zoom in", "<control>X", dw_request_zoom, 0, NULL},
  {"/_Options", NULL, NULL, 0, "<Branch>"},
  {"/_Help", NULL, NULL, 0, "<LastBranch>"},
  {"/_Help/About", NULL, dw_about, 0, NULL},
};

static void
make_menu (Instance i, Draw_Window dw)
{
  GtkItemFactory *item_factory;
  GtkAccelGroup *accel_group;
  gint nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);

  accel_group = gtk_accel_group_new ();

  item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>",
                                       accel_group);

  gtk_item_factory_create_items (item_factory, nmenu_items, menu_items, dw);

  gtk_window_add_accel_group (GTK_WINDOW (i->window), accel_group);

  dw->menubar = gtk_item_factory_get_widget (item_factory, "<main>");
}


dw_make_fs(Draw_Window dw)
{
  dw->fileselector = gtk_file_selection_new ("");
  gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (dw->fileselector));
  gtk_widget_hide (dw->fileselector);


  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (dw->fileselector)->ok_button),
                      "clicked", (GtkSignalFunc) dw_fs_ok, dw);

  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (dw->fileselector)->cancel_button),
                      "clicked", (GtkSignalFunc) dw_fs_cancel, dw);

  gtk_signal_connect (GTK_OBJECT (dw->fileselector),
                   "delete-event", (GtkSignalFunc) dw_fs_delete_cancel, dw);

  dw->fscb = (GtkSignalFunc) dw_fs_cancel;
}


Draw_Window
draw_window_create (Instance i)
{
  Draw_Window dw = (Draw_Window) malloc (sizeof (struct draw_window_struct));

  bzero (dw, sizeof (struct draw_window_struct));

  dw->drawing = i->dr;
  dw->mouse = i->dr->m;;

  dw->wt = transform_create ();
  dw->vt = transform_create ();
  dw->t = transform_create ();
  dw->it = transform_create ();

  transform_null (dw->wt);
  transform_null (dw->t);
  transform_null (dw->it);

  transform_copy (dw->vt, i->dr->d->wholepage);

  dw->vbox = gtk_vbox_new (FALSE, 0);

  make_menu (i, dw);
  gtk_box_pack_start (GTK_BOX (dw->vbox), dw->menubar, FALSE, FALSE, 0);

  dw->hbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (dw->vbox), dw->hbox, FALSE, FALSE, 0);


  dw->label = gtk_label_new ("Annotation:");
  gtk_box_pack_start (GTK_BOX (dw->hbox), dw->label, FALSE, FALSE, 0);

  dw->annotation = gtk_entry_new_with_max_length (256);

  gtk_signal_connect (GTK_OBJECT (dw->annotation), "activate",
                      GTK_SIGNAL_FUNC (annotate_callback),
                      dw);
  gtk_signal_connect (GTK_OBJECT (dw->annotation), "changed",
                      GTK_SIGNAL_FUNC (annotate_callback),
                      dw);


  gtk_box_pack_start (GTK_BOX (dw->hbox), dw->annotation, TRUE, TRUE, 0);

  dw->table = gtk_table_new (2, 2, FALSE);
  gtk_box_pack_start (GTK_BOX (dw->vbox), dw->table, TRUE, TRUE, 0);

  dw->area = gtk_drawing_area_new ();

  gtk_table_attach (GTK_TABLE (dw->table), dw->area, 1, 2, 1, 2,
                    GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);

  dw->hrule = gtk_hruler_new ();
  gtk_ruler_set_metric (GTK_RULER (dw->hrule), GTK_PIXELS);
  gtk_ruler_set_range (GTK_RULER (dw->hrule), 0, 1, 0, 1);

  gtk_table_attach (GTK_TABLE (dw->table), dw->hrule, 1, 2, 0, 1,
                    GTK_EXPAND | GTK_SHRINK | GTK_FILL, GTK_FILL, 0, 0);

  dw->vrule = gtk_vruler_new ();

  gtk_ruler_set_metric (GTK_RULER (dw->vrule), GTK_PIXELS);
  gtk_ruler_set_range (GTK_RULER (dw->vrule), 0, 1, 0, 1);

  gtk_table_attach (GTK_TABLE (dw->table), dw->vrule, 0, 1, 1, 2,
                    GTK_FILL, GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);

  gtk_signal_connect (GTK_OBJECT (dw->area), "expose_event",
                      (GtkSignalFunc) expose_event, dw);
  gtk_signal_connect (GTK_OBJECT (dw->area), "configure_event",
                      (GtkSignalFunc) configure_event, dw);
  gtk_signal_connect (GTK_OBJECT (dw->area), "motion_notify_event",
                      (GtkSignalFunc) motion_notify_event, dw);
  gtk_signal_connect (GTK_OBJECT (dw->area), "button_press_event",
                      (GtkSignalFunc) button_event, dw);
  gtk_signal_connect (GTK_OBJECT (dw->area), "button_release_event",
                      (GtkSignalFunc) button_event, dw);

  gtk_widget_set_events (dw->area, GDK_EXPOSURE_MASK
                         | GDK_LEAVE_NOTIFY_MASK
                         | GDK_BUTTON_PRESS_MASK
                         | GDK_BUTTON_RELEASE_MASK
                         | GDK_POINTER_MOTION_MASK
                         | GDK_POINTER_MOTION_HINT_MASK);

  gtk_widget_show (dw->menubar);
  gtk_widget_show (dw->annotation);
  gtk_widget_show (dw->label);
  gtk_widget_show (dw->hbox);
  gtk_widget_show (dw->area);
  gtk_widget_show (dw->hrule);
  gtk_widget_show (dw->vrule);
  gtk_widget_show (dw->table);

  dw_make_fs(dw);

  hide_annotate (dw);

  dw->gc = Null;

  sortout_windowsize (dw);


  dw->digit_id = digitizer_register_eventproc (i->dr->d, puck_motion, (void *) dw);
  dw->drawing_id = drawing_register_eventproc (i->dr, drawing_event, (void *) dw);


  return dw;
}
