


/*
 * editor.c:
 *
 * Copyright (c) 1999 James McKenzie <james@fishsoup.dhs.org>.
 * All rights reserved.
 *
 * $Log: editor.c,v $
 * Revision 1.8  2000/03/11 18:34:56  root
 * #make_checkin
 *
 * Revision 1.7  1999/11/22 04:43:40  root
 * #make_checkin
 *
 * Revision 1.6  1999/11/22 04:36:00  root
 * #make_checkin
 *
 * Revision 1.5  1999/11/21 19:48:05  root
 * #make_checkin
 *
 * Revision 1.4  1999/11/21 19:23:06  root
 * #make_checkin
 *
 * Revision 1.3  1999/11/21 05:26:48  root
 * #make_checkin
 *
 * Revision 1.2  1999/11/21 02:56:20  root
 * #make_checkin
 *
 * Revision 1.1  1999/11/21 01:08:34  root
 * #make_checkin
 *
 *
 */

static char rcsid[] = "$Id: editor.c,v 1.8 2000/03/11 18:34:56 root Exp root $";

#include "project.h"

static int callback_id = 1;

int
drawing_register_eventproc (Drawing d, Drawing_EventProc f, void *ptr)
{
  Drawing_Callback cb;

  cb = CREATE_OBJ (Drawing_Callback);
  cb->id = callback_id++;
  cb->func = f;
  cb->ptr = ptr;

  Obj_Insert (d->callbacks, cb, ObjInsertTail);

  return cb->id;
}


int
drawing_unregister_eventproc (Drawing d, int id)
{
  Drawing_Callback cb = d->callbacks->head;

  while (cb)
    {

      if (cb->id == id)
        {
          Obj_Remove (d->callbacks, cb);
          Obj_Delete (cb);
          return 1;
        }

      cb = cb->next;
    }


  gripe (GRIPE_WARN, "drawing_unregister_eventproc: can't find id %d\n", id);

  return 0;
}


void
drawing_destructor (Drawing d)
{
  mouse_unregister_eventproc (d->m, d->mouse_cb_id);
  digitizer_unregister_eventproc (d->d, d->dig_cb_id);
  if (d->current)
    Obj_Delete (d->current);
  d->current = Null;
  Obj_Delete (d->lines);

  if (d->callbacks)
    Obj_Delete (d->callbacks);

  digit_free (d);
}


dispatch_line_drawing_event (Drawing dr, Drawing_Event_Type t, Line l)
{
  Drawing_Event e;
  Drawing_Callback cb = dr->callbacks->head;

  e.type = t;
  e.d.l = l;

  while (cb)
    {
      (cb->func) (dr, &e, cb->ptr);
      cb = cb->next;
    }

}

static void
insert_current (Drawing dr)
{
  if (!dr->current)
    return;

  line_pack (dr->current);

  Obj_Insert (dr->lines, dr->current, ObjInsertTail);

  dispatch_line_drawing_event (dr, DRAW_LINEDONE, dr->current);
  if (dr->selected)
    {
      dispatch_line_drawing_event (dr, DRAW_UNSELLINE, dr->selected);
    }
  dr->selected = dr->current;
  dispatch_line_drawing_event (dr, DRAW_SELLINE, dr->selected);


  dr->current = Null;

}

static void
digitizer_callback (Digitizer d, Digitizer_Event * e, void *ptr)
{
  Drawing dr = (Drawing) ptr;
  Point p =
  {e->x, e->y};

  if (e->state & DIGIT_BUTTON (0))
    {

      if (!dr->current)
        {                       /* New line */
          dr->current = CREATE_OBJ (Line);
          dr->current->type = LINE_LINE;
          dispatch_line_drawing_event (dr, DRAW_NEWLINE, dr->current);
        }

      line_addpt (dr->current, p);
      dispatch_line_drawing_event (dr, DRAW_LINEADD, dr->current);

    }
  else
    {
      if (dr->current)
        insert_current (dr);
    }

  if ((e->type == DIGITIZER_BUTTDOWN) && (e->button == 2))
    {
      if (dr->current)
        insert_current (dr);
      dr->current = CREATE_OBJ (Line);
      dr->current->type = LINE_POINT;

      dispatch_line_drawing_event (dr, DRAW_NEWLINE, dr->current);
      line_addpt (dr->current, p);
      insert_current (dr);



    }

}

#define BIGNUMBER 1.0e5

static void
go_find_and_select (Drawing dr, float x, float y)
{
  float sqd = BIGNUMBER;
  Line l;
  int n;
  Point *p;

  if (dr->selected)
    {
      dispatch_line_drawing_event (dr, DRAW_UNSELLINE, dr->selected);
      dr->selected = Null;
    }

  l = dr->lines->head;

  while (l)
    {
      p = l->l;
      n = l->n;
      while (n--)
        {
          float d = ((p->x - x) * (p->x - x)) + ((p->y - y) * (p->y - y));

          if (d < sqd)
            {
              sqd = d;
              dr->selected = l;
            }
          p++;
        }
      l = l->next;
    }

  if (dr->selected)
    {
      dispatch_line_drawing_event (dr, DRAW_SELLINE, dr->selected);
    }

}


void
drawing_delete_selected (Drawing d)
{
  if (!d->selected)
    return;
  dispatch_line_drawing_event (d, DRAW_DELLINE, d->selected);

  Obj_Remove (d->lines, d->selected);
  Obj_Delete (d->selected);
  d->selected = Null;

}

static void
mouse_callback (Mouse m, Mouse_Event * e, void *ptr)
{
  Drawing dr = (Drawing) ptr;
  if ((e->type == MOUSE_BUTTDOWN) && (e->button == 0))
    {
      go_find_and_select (dr, e->x, e->y);
    }
}


Drawing
drawing_create (Mouse m, Digitizer d)
{
  Drawing dr = CREATE_OBJ (Drawing);

  dr->m = m;
  dr->d = d;

  dr->callbacks = CREATE_LIST (Drawing_Callback);

  dr->mouse_cb_id = mouse_register_eventproc (m, mouse_callback, dr);
  dr->dig_cb_id = digitizer_register_eventproc (d, digitizer_callback, dr);

  dr->lines = CREATE_LIST (Line);

  return dr;
}

void
drawing_flush (Drawing d)
{
  if (d->current)
    {
      Obj_Delete (d->current);
      d->current = Null;
    }
  if (d->selected)
    {
      dispatch_line_drawing_event (d, DRAW_UNSELLINE, d->selected);
      d->selected = Null;
    }

  Obj_Delete (d->lines);
  d->lines = CREATE_LIST (Line);

  dispatch_line_drawing_event (d, DRAW_NEWDRAWING, Null);

}

void
drawing_write (Drawing d, FILE * s)
{
  Line l = d->lines->head;

  magic_write (DRAWING_MAGIC, s);

  while (l)
    {
      line_write (l, s);
      l = l->next;
    }

  magic_write (EOF_MAGIC, s);
}

int
drawing_load (Drawing d, FILE * s)
{
  Line l;

  drawing_flush (d);

  if (!magic_check (DRAWING_MAGIC, s))
    {
      gripe (GRIPE_WARN, "drawing_load: wrong magic number");
      return 0;
    }

  while (!magic_check (EOF_MAGIC, s))
    {
      if (l = line_read (s))
        {
          Obj_Insert (d->lines, l, ObjInsertTail);
        }
    }

  dispatch_line_drawing_event (d, DRAW_NEWDRAWING, Null);

  return 1;

}
