
/*
 * mouse.c:
 *
 * Copyright (c) 1999 James McKenzie <james@fishsoup.dhs.org>.
 * All rights reserved.
 *
 * $Log: mouse.c,v $
 * Revision 1.3  1999/11/21 05:26:48  root
 * #make_checkin
 *
 * Revision 1.2  1999/11/21 02:56:20  root
 * #make_checkin
 *
 * Revision 1.1  1999/11/21 01:08:34  root
 * #make_checkin
 *
 *
 */

static char rcsid[] = "$Id: mouse.c,v 1.3 1999/11/21 05:26:48 root Exp $";

#include "project.h"

static int callback_id = 1;

static void
mouse_send_event (Mouse m, Mouse_Event * e)
{

  Mouse_Callback cb = m->callbacks->head;

  while (cb)
    {
      (cb->func) (m, e, cb->ptr);
      cb = cb->next;
    }

}

void
mouse_inject_state (Mouse m, float x, float y, int state)
{
  Mouse_Event e;
  int i;

  e.x = x;
  e.y = y;
  e.state = state;
  e.button = 0;

  if (m->state == state)
    {
      if ((m->x != x) || (m->y != y))
        {

          m->x = x;
          m->y = y;

          e.type = (state & MOUSE_BUTTONS) ? MOUSE_DRAG : MOUSE_MOTION;
          mouse_send_event (m, &e);

          return;
        }

    }

  for (i = 0; i < MOUSE_NUMBUTTONS; ++i)
    {

      if ((m->state & MOUSE_BUTTON (i)) != (state & MOUSE_BUTTON (i)))
        {
          e.type = (state & MOUSE_BUTTON (i)) ? MOUSE_BUTTDOWN : MOUSE_BUTTUP;
          e.button = i;

          m->state &= ~MOUSE_BUTTON (i);
          m->state |= state & MOUSE_BUTTON (i);
          mouse_send_event (m, &e);
          return;

        }


    }

}

void
mouse_destroy (Mouse m)
{
  Obj_Delete (m->callbacks);
  free (m);
}

Mouse
mouse_create ()
{
  Mouse m = (Mouse) digit_malloc (sizeof (struct mouse_struct));
  m->callbacks = CREATE_LIST (Mouse_Callback);
  return m;
}

int
mouse_register_eventproc (Mouse d, Mouse_EventProc f, void *ptr)
{
  Mouse_Callback cb;

  cb = CREATE_OBJ (Mouse_Callback);
  cb->id = callback_id++;
  cb->func = f;
  cb->ptr = ptr;

  Obj_Insert (d->callbacks, cb, ObjInsertTail);

  return cb->id;
}


int
mouse_unregister_eventproc (Mouse d, int id)
{
  Mouse_Callback cb = d->callbacks->head;

  while (cb)
    {

      if (cb->id == id)
        {
          Obj_Remove (d->callbacks, cb);
          Obj_Delete (cb);
          return 1;
        }

      cb = cb->next;
    }


  gripe (GRIPE_WARN, "mouse_unregister_eventproc: can't find id %d\n", id);

  return 0;
}
