/*
 * transform.c:
 *
 * Copyright (c) 1999 James McKenzie <james@fishsoup.dhs.org>.
 * All rights reserved.
 *
 * $Log: transform.c,v $
 * Revision 1.9  1999/11/22 04:36:00  root
 * #make_checkin
 *
 * Revision 1.8  1999/11/21 19:48:05  root
 * #make_checkin
 *
 * Revision 1.7  1999/11/21 19:23:06  root
 * #make_checkin
 *
 * Revision 1.6  1999/11/21 01:08:34  root
 * #make_checkin
 *
 * Revision 1.5  1999/11/20 18:47:30  root
 * #make_checkin
 *
 * Revision 1.4  1999/11/20 13:43:30  root
 * #make_checkin
 *
 * Revision 1.3  1999/11/19 00:27:46  root
 * #make_checkin
 *
 * Revision 1.2  1999/11/19 00:26:09  root
 * #make_checkin
 *
 * Revision 1.1  1999/11/19 00:25:33  root
 * #make_checkin
 *
 *
 */

static char rcsid[] = "$Id: transform.c,v 1.9 1999/11/22 04:36:00 root Exp $";

#include "project.h"

Transform
transform_create ()
{
  Transform ret;

  ret = digit_malloc (sizeof (struct transform_struct));

  return (ret);
}

void
transform_free (Transform t)
{
  digit_free (t);
}


void
transform_null (Transform t)
{
  t->a = t->d = 1.0;
  t->b = t->c = t->e = t->f = 0.0;
}

void
transform_concat (Transform r, Transform a, Transform b)
{
  r->a = (a->a * b->a) + (a->b * b->c);
  r->b = (a->a * b->b) + (a->b * b->d);
  r->c = (a->c * b->a) + (a->d * b->c);
  r->d = (a->c * b->b) + (a->d * b->d);
  r->e = a->e + (a->a * b->e) + (a->b * b->f);
  r->f = a->f + (a->c * b->e) + (a->d * b->f);
}

void
transform_copy (Transform d, Transform s)
{

  bcopy (s, d, sizeof (struct transform_struct));

}

Transform
transform_dup (Transform t)
{
  Transform r = transform_create ();

  transform_copy (r, t);

  return r;
}

void
transform_invert (Transform r, Transform t)
{

  float det = 1.0 / ((t->a * t->d) - (t->b * t->c));


  r->a = det * t->d;
  r->b = -det * t->b;
  r->c = -det * t->c;
  r->d = det * t->a;

  r->e = -(r->a * t->e) - (r->b * t->f);
  r->f = -(r->c * t->e) - (r->d * t->f);


}

/* Assemble a transform that maps the square (0,0)-(1,1) unto the window */
/* (0,h)-(w,0) [ie including the coord flip] */

Transform
transform_window (float w, float h)
{
  Transform r = transform_create ();
  float m = w < h ? w : h;

  r->a = m;
  r->b = 0;
  r->c = 0;
  r->d = -m;
  r->e = 0;
  r->f = h;

  return r;
}

void
transform_from_zoom (Transform r, float x1, float y1, float x2, float y2)
{
  float s, t;

  if (x1 > x2)
    {
      t = x2;
      x2 = x1;
      x1 = t;
    }
  if (y1 > y2)
    {
      t = y2;
      y2 = y1;
      y1 = t;
    }


  s = (x2 - x1);
  t = (y2 - y1);

  if (s < t)
    s = t;


  transform_null (r);

  r->a = r->d = 1.0 / s;
  r->e = -x1 / s;
  r->f = -y1 / s;

}

void
transform_print (Transform t)
{
  printf ("((%f %f),(%f %f),(%f %f))\n",
          t->a, t->b, t->c, t->d, t->e, t->f);

}
